// Variables
const llmHost = 'https://llm.vfsh.me';
const generateEndpoint = '/api/generate';

// Grab elements from the page
const userInput = document.getElementById('userInput');
const sendButton = document.getElementById('sendButton');
const outputArea = document.getElementById('outputArea');
const loadingSpinner = document.getElementById('loadingSpinner');
const darkModeToggle = document.getElementById('darkModeToggle');

// Functions
async function sendGen() {
    loadingSpinner.classList.remove('hidden');
    outputArea.innerHTML = '<span class="text-blue-500">Generating... (please be patient, streaming responses are not enabled yet...)</span>';
    const response = await fetch(`${llmHost}${generateEndpoint}`, {
        method: 'POST',
        headers: {
            'Content-Type': 'application/json'
        },
        body: JSON.stringify({ model: 'llama3.2', prompt: userInput.value })
    });

    return response;
}

async function readResponse(response) {
    const reader = response.body.getReader();
    const decoder = new TextDecoder('utf-8');

    try {
        let finishedResponse = '';
        let JSONResponse = new Array();
        let cleanResponse = new Array();
        function read() {
            // Recursive function to read the stream
            reader.read().then(( chunk ) => {
                if (chunk.value) {
                    const text = decoder.decode(chunk.value);
                    finishedResponse += text;
                }
                if (chunk.done) {
                    const lines = finishedResponse.split('\n');
                    for (let i = 0; i < lines.length; i++) {
                        const line = lines[i];
                        if (line.length > 0) {
                            const JSONline = JSON.parse(line);
                            JSONResponse.push(JSONline);
                        }
                    }
                    for (let i = 0; i < JSONResponse.length; i++) {
                        const JSONline = JSONResponse[i];
                        if (JSONline.response) {
                            cleanResponse.push(JSONline.response);
                        }
                    }
                    loadingSpinner.classList.add('hidden');
                    outputArea.innerHTML = cleanResponse.join('');
                    return;
                }
                read();
            });
        }
        read();
    } catch (error) {
        outputArea.innerHTML = `<span class="text-red-500">Error: ${error.message}</span>`;
    }
}


darkModeToggle.addEventListener('click', () => {
    document.body.classList.toggle('dark');
});

// Event listeners
sendButton.addEventListener('click', async () => {
    outputArea.innerHTML = '';
    const response = await sendGen();
    readResponse(response);
});